% 
% function run_cppca()
% 
%   This function runs the CPPCA algorithm.
%

%
% CPPCA-PR: CPPCA with Partitioned-Residual Reconstruction
% Copyright (C) 2012  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%

function run_cppca()

addpath(genpath('../CPPCA-3.3-1'));
addpath(genpath('../BCS/MT_CS_demo'));

% num_rows = 64;
% num_cols = 64;
% N = 169;
% filename = 'hydice';
% anomaly_percentage = 0.00463;

num_rows = 200;
num_cols = 200;
N = 158;
filename = 'lcvf';
anomaly_percentage = 2/num_rows/num_cols;

M = num_rows * num_cols;

fid = fopen([filename '.dat'], 'r', 'ieee-be');
X = fread(fid, [N M], 'float32');
fclose(fid);

fid = fopen([filename '.ground_truth.dat'], 'r', 'ieee-be');
ground_truth_map2 = fread(fid, [1 M], 'uint8');
fclose(fid);

[N M] = size(X);
J = 20;
projection_matrix_file = ['projections.' num2str(N) '.' num2str(J) '.mat'];

relative_dimensions = [0.1 0.15 0.2 0.3 0.4 0.5];

Ks = round(relative_dimensions * N);

Ls_normal = round(Ks ./ log(N));

X_original = X;

disp('Running RX...');
tic
D = RX(X);
toc
tau = RX_threshold(D, 1 - anomaly_percentage);
anomaly_map2 = (D > tau);
for j = 1:J
  ground_truth_map{j} = logical(ground_truth_map2(j:J:M));
  anomaly_map{j} = ground_truth_map{j};
  normal_map{j} = ~anomaly_map{j};
end

disp(['Number of anomalies: ' num2str(sum(ground_truth_map2))]);
disp(['Number of RX-detected anomalies: ' num2str(sum(anomaly_map2))]);
disp(['Percent correct = ' num2str((M - ...
	sum(xor(anomaly_map2, ground_truth_map2))) / M * 100)]);

imwrite(uint8(255*reshape(anomaly_map2, [num_cols num_rows])'), ...
    [filename '.anomaly.pgm']);

disp('Running Projections...');
for index1 = 1:length(Ks)
  
  K = Ks(index1);
  
  P{index1} = CPPCA_GenerateProjections(N, K, J, projection_matrix_file);

  [Y_tilde{index1}, X] = CPPCA_Encoder(X_original, P{index1});
end  

disp('Running CPPCA...');
for index1 = 1:length(Ks)
  K = Ks(index1);
  L = Ls_normal(index1);

  tic
  [X_check_CPPCA, Psi_CPPCA] = CPPCA_Decoder(Y_tilde{index1}, ...
      P{index1}, L);
  toc

  D_CPPCA_normal(index1) = SNR_PartitionedDataset(X, X_check_CPPCA, ...
      normal_map, [Psi_CPPCA zeros(N, N - L)]);
  S_CPPCA_normal(index1) = SAM_PartitionedDataset(X, X_check_CPPCA, ...
      normal_map, [Psi_CPPCA zeros(N, N - L)]);
  D_CPPCA_anomaly(index1) = SNR_PartitionedDataset(X, X_check_CPPCA, ...
      anomaly_map, [Psi_CPPCA zeros(N, N - L)]);
  S_CPPCA_anomaly(index1) = SAM_PartitionedDataset(X, X_check_CPPCA, ...
      anomaly_map, [Psi_CPPCA zeros(N, N - L)]);
end

save([filename '.cppca.results.mat'], ...
      'relative_dimensions', ...
      'D_CPPCA_normal', 'S_CPPCA_normal', ...
      'D_CPPCA_anomaly', 'S_CPPCA_anomaly');

disp(['CPPCA normal: ' num2str(D_CPPCA_normal) ' dB']);
disp(['CPPCA anomaly: ' num2str(D_CPPCA_anomaly) ' dB']);
disp(['CPPCA normal: ' num2str(S_CPPCA_normal) ' degrees']);
disp(['CPPCA anomaly: ' num2str(S_CPPCA_anomaly) ' degrees']);
